@@ -86,6 +86,17 @@ class behat_hooks extends behat_base {
      */
     protected static $currentstepexception = null;
 
+    /**
+     * If an Exception is thrown in the BeforeScenario hook it will cause the Scenario to be skipped, and the exit code
+     * to be non-zero triggering a potential rerun.
+     *
+     * To combat this the exception is stored and re-thrown when looking for exceptions.
+     * This allows the test to instead be failed and re-run correctly.
+     *
+     * @var null|Exception
+     */
+    protected static $currentscenarioexception = null;
+
     /**
      * If we are saving any kind of dump on failure we should use the same parent dir during a run.
      *
@@ -166,7 +177,7 @@ class behat_hooks extends behat_base {
             $message = <<<EOF
 Your behat test site is outdated, please run the following command from your Moodle dirroot to drop, and reinstall the Behat test site.
 
-    {$comandpath}
+    {$commandpath}
 
 EOF;
             self::log_and_stop($message);
@@ -296,57 +307,6 @@ EOF;
         $this->restart_session();
     }
 
-    /**
-     * Start the session before the first javascript scenario.
-     *
-     * This is treated slightly differently to try to capture when Selenium is not running at all.
-     *
-     * @BeforeScenario @javascript
-     * @param BeforeScenarioScope $scope scope passed by event fired before scenario.
-     */
-    public function before_first_scenario_start_session(BeforeScenarioScope $scope) {
-        if (!self::is_first_javascript_scenario()) {
-            // The first Scenario has started.
-            // The `before_subsequent_scenario_start_session` function will restart the session instead.
-            return;
-        }
-        self::$firstjavascriptscenarioseen = true;
-
-        $docsurl = behat_command::DOCS_URL;
-        $driverexceptionmsg = <<<EOF
-
-The Selenium or WebDriver server is not running. You must start it to run tests that involve Javascript.
-See {$docsurl} for more information.
-
-The following debugging information is available:
-
-EOF;
-
-
-        try {
-            $this->restart_session();
-        } catch (CurlExec $e) {
-            // The CurlExec Exception is thrown by WebDriver.
-            self::log_and_stop(
-                $driverexceptionmsg . '. ' .
-                $e->getMessage() . "\n\n" .
-                format_backtrace($e->getTrace(), true)
-            );
-        } catch (DriverException $e) {
-            self::log_and_stop(
-                $driverexceptionmsg . '. ' .
-                $e->getMessage() . "\n\n" .
-                format_backtrace($e->getTrace(), true)
-            );
-        } catch (UnknownError $e) {
-            // Generic 'I have no idea' Selenium error. Custom exception to provide more feedback about possible solutions.
-            self::log_and_stop(
-                $e->getMessage() . "\n\n" .
-                format_backtrace($e->getTrace(), true)
-            );
-        }
-    }
-
     /**
      * Start the session before each javascript scenario.
      *
@@ -361,8 +321,13 @@ EOF;
             // The `before_first_scenario_start_session` function will have started the session instead.
             return;
         }
+        self::$currentscenarioexception = null;
 
-        $this->restart_session();
+        try {
+            $this->restart_session();
+        } catch (Exception $e) {
+            self::$currentscenarioexception = $e;
+        }
     }
 
     /**
@@ -373,6 +338,12 @@ EOF;
      */
     public function before_scenario_hook(BeforeScenarioScope $scope) {
         global $DB;
+        if (self::$currentscenarioexception) {
+            // A BeforeScenario hook triggered an exception and marked this test as failed.
+            // Skip this hook as it will likely fail.
+            return;
+        }
+
         $suitename = $scope->getSuite()->getName();
 
         // Register behat selectors for theme, if suite is changed. We do it for every suite change.
@@ -443,6 +414,16 @@ EOF;
         $this->resize_window('medium');
     }
 
+    /**
+     * Mark the first Javascript Scenario as have been seen.
+     *
+     * @BeforeScenario
+     * @param BeforeScenarioScope $scope scope passed by event fired before scenario.
+     */
+    public function mark_first_js_scenario_as_seen(BeforeScenarioScope $scope) {
+        self::$firstjavascriptscenarioseen = true;
+    }
+
     /**
      * Hook to open the site root before the first step in the suite.
      * Yes, this is in a strange location and should be in the BeforeScenario hook, but failures in the test setUp lead
@@ -464,7 +445,7 @@ EOF;
             // Again, this would be better in the BeforeSuite hook, but that does not have access to the selectors in
             // order to perform the necessary searches.
             $session = $this->getSession();
-            $session->visit($this->locate_path('/'));
+            $this->execute('behat_general::i_visit', ['/']);
 
             // Checking that the root path is a Moodle test site.
             if (self::is_first_scenario()) {
@@ -495,6 +476,12 @@ EOF;
      * @BeforeStep
      */
     public function before_step_javascript(BeforeStepScope $scope) {
+        if (self::$currentscenarioexception) {
+            // A BeforeScenario hook triggered an exception and marked this test as failed.
+            // Skip this hook as it will likely fail.
+            return;
+        }
+
         self::$currentstepexception = null;
 
         // Only run if JS.
@@ -711,6 +698,11 @@ EOF;
      * @see Moodle\BehatExtension\EventDispatcher\Tester\ChainedStepTester
      */
     public function i_look_for_exceptions() {
+        // If the scenario already failed in a hook throw the exception.
+        if (!is_null(self::$currentscenarioexception)) {
+            throw self::$currentscenarioexception;
+        }
+
         // If the step already failed in a hook throw the exception.
         if (!is_null(self::$currentstepexception)) {
             throw self::$currentstepexception;
